import { _decorator, Component, Node, UI } from 'cc';
const { ccclass, property } = _decorator;

let _walk = UI.prototype.walk
UI.prototype.walk = function (node: Node, level = 0): void {
    const uiProps = node._uiProps;
    /**
     * @en Only changed here
     * @zn 只改了这里
     */
    if (uiProps["RenderChildSort"]) {
        let childs = uiProps["RenderChildSort"].getRenderChilds()
        for (let i = 0; i < childs.length; ++i) {
            let _node = childs[i]
            let _static = _node._static
            _node._static = true
            _walk.call(this, _node, level)
            _node._static = _static
        }
    }
    else
        _walk.call(this, node, level)
}

@ccclass('RenderChildSort')
export class RenderChildSort extends Component {

    private childs: Node[] = []
    private childDeaths: number[] = []

    onEnable(): void {
        this.node._uiProps["RenderChildSort"] = this
    }

    onDisable(): void {
        this.node._uiProps["RenderChildSort"] = null
    }

    /**
     * @en Rendering all child nodes
     * @zn 渲染所有子节点
     */
    public getRenderChilds() {
        this.childs.length = 0
        this.childDeaths.length = 0
        // 可以剔除屏幕外 不需要的节点 Can eliminate unnecessary nodes outside the screen
        for (let i = 0; i < this.node.children.length; i++)
            this._updateRenderChild(this.node.children[i])
        return this.childs
    }

    /** */
      /**
     * @en Same node combination
     * @zn 相同节点组合
     */
    private _updateRenderChild(child: Node) {
        if (!child.activeInHierarchy)
            return
        if (child._uiProps.opacity == 0)
            return
        let level = 0
        let parent: Node = child.parent
        while (parent) {
            if (!parent._uiProps["RenderChildSort"])
                level++
            else
                break
            parent = parent.parent
        }

        let index = child.getSiblingIndex()
        let deathNum = level * 1000 + index

        let deathIndex = 0
        for (deathIndex; deathIndex < this.childDeaths.length; deathIndex++)
            if (deathNum <= this.childDeaths[deathIndex])
                break
        this.childDeaths.splice(deathIndex, 0, deathNum)
        this.childs.splice(deathIndex, 0, child)

        for (let i = 0; i < child.children.length; i++)
            this._updateRenderChild(child.children[i])
    }

}


